import {throwError as observableThrowError, of as observableOf,  Observable } from 'rxjs';
import { Ng2IzitoastService } from 'ng2-izitoast';
import { async, ComponentFixture, TestBed } from '@angular/core/testing';
import { SharedModule, ResourceService, ConfigService, IAction } from '@sunbird/shared';
import { CoreModule, LearnerService, CoursesService, SearchService, OrgDetailsService} from '@sunbird/core';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRoute, Router } from '@angular/router';
import * as _ from 'lodash';
import { NO_ERRORS_SCHEMA } from '@angular/core';
import { ExploreContentComponent } from './explore-content.component';
import { Response } from './explore-content.component.spec.data';
import { TelemetryModule } from '@sunbird/telemetry';
import { PublicPlayerService } from './../../../../services';
describe('ExploreContentComponent', () => {
  let component: ExploreContentComponent;
  let fixture: ComponentFixture<ExploreContentComponent>;
  const resourceBundle = {
    'messages': {
      'stmsg': {
        'm0007': 'Search for something else',
        'm0006': 'No result'
      },
      'fmsg': {
        'm0077': 'Fetching search result failed',
        'm0051': 'Fetching other courses failed, please try again later...'
      }
    }
  };
  const mockQueryParma = {
    'query': 'hello'
  };
  class RouterStub {
    navigate = jasmine.createSpy('navigate');
  }
  const fakeActivatedRoute = {
    'params': observableOf({ pageNumber: '3' }),
    'queryParams': observableOf({
      sortType: 'desc', sort_by: 'lastUpdatedOn',
      key: 'hello'
    }),
    snapshot: {
      params: {
        slug: 'ap'
      },
      data: {
        telemetry: {
          env: 'get', pageid: 'get', type: 'edit', subtype: 'paginate'
        }
      }
    }
  };
  beforeEach(async(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, SharedModule.forRoot(), CoreModule.forRoot(), TelemetryModule.forRoot()],
      declarations: [ExploreContentComponent],
      providers: [ConfigService, SearchService, LearnerService, OrgDetailsService,
        { provide: ResourceService, useValue: resourceBundle },
        { provide: Router, useClass: RouterStub },
        { provide: ActivatedRoute, useValue: fakeActivatedRoute }, PublicPlayerService],
      schemas: [NO_ERRORS_SCHEMA]
    })
      .compileComponents();
  }));
  beforeEach(() => {
    fixture = TestBed.createComponent(ExploreContentComponent);
    component = fixture.componentInstance;
  });
  it('should subscribe to searchService', () => {
    component.slug = '123456567';
    component.queryParams = mockQueryParma;
    component.exploreRoutingUrl = 'explore';
    const searchService = TestBed.get(SearchService);
    const orgManagementService = TestBed.get(OrgDetailsService);
    component.dataDrivenFilter = {};
    spyOn(searchService, 'contentSearch').and.callFake(() => observableOf(Response.successData));
    component.searchList = Response.successData.result.content;
    component.populateContentSearch();
    expect(component.queryParams.sortType).toString();
    expect(component.showLoader).toBeFalsy();
    expect(component.searchList).toBeDefined();
    expect(component.totalCount).toBeDefined();
  });

  it('should call searchService with badgeAssertions and channel', () => {
    component.slug = '123456567';
    component.hashTagId = '0123166367624478721';
    component.queryParams = {
      'key': 'hello'
    };
    component.filters = {
      contentType: ['Collection', 'TextBook', 'LessonPlan', 'Resource', 'Story', 'Worksheet', 'Game']
    };
    component.dataDrivenFilter = {};
    component.dataDrivenFilter['board'] = ['CBSE'];
    const config = TestBed.get(ConfigService);
    const requestParams = Response.requestParam;
    requestParams['params'] = config.appConfig.ExplorePage.contentApiQueryParams;
    const searchService = TestBed.get(SearchService);
    const orgManagementService = TestBed.get(OrgDetailsService);
    spyOn(searchService, 'contentSearch').and.callFake(() => observableOf(Response.successData));
    component.searchList = Response.successData.result.content;
    component.populateContentSearch();
    expect(searchService.contentSearch).toHaveBeenCalledWith(requestParams);
    expect(component.queryParams.sortType).toString();
    expect(component.showLoader).toBeFalsy();
    expect(component.searchList).toBeDefined();
    expect(component.totalCount).toBeDefined();
  });
  it('should throw error when searchService api throw error ', () => {
    component.slug = '123456567';
    const searchService = TestBed.get(SearchService);
    const orgManagementService = TestBed.get(OrgDetailsService);
    component.dataDrivenFilter = {};
    spyOn(searchService, 'contentSearch').and.callFake(() => observableThrowError({}));
    component.queryParams = mockQueryParma;
    component.populateContentSearch();
    expect(component.showLoader).toBeFalsy();
    expect(component.noResult).toBeTruthy();
  });
  it('when count is 0 should show no result found', () => {
    component.slug = '123456567';
    const searchService = TestBed.get(SearchService);
    const orgManagementService = TestBed.get(OrgDetailsService);
    component.dataDrivenFilter = {};
    spyOn(searchService, 'contentSearch').and.callFake(() => observableOf(Response.noResult));
    component.searchList = Response.noResult.result.content;
    component.totalCount = Response.noResult.result.count;
    component.queryParams = mockQueryParma;
    component.populateContentSearch();
    expect(component.showLoader).toBeFalsy();
  });
  it('should call compareObjects method', () => {
    const objA = {
      board: ['gradeLevel']
    };
    const objB = {
      board: ['gradeLevel']
    };
    component.compareObjects(objA, objB);
    expect(component.facets).toEqual(undefined);
  });
  it('should call navigateToPage method', () => {
    const router = TestBed.get(Router);
    const page = 3;
    component.pager = {
      currentPage: 3,
      endIndex: 19,
      endPage: 5,
      pageSize: 20,
      pages: [1, 2, 3, 4],
      startIndex: 0,
      startPage: 1,
      totalItems: 45,
      totalPages: 66
    };
    component.exploreRoutingUrl = 'explore';
    component.queryParams = { key: 'abc' };
    component.navigateToPage(page);
    expect(component.pageNumber).toEqual(page);
    expect(router.navigate).toHaveBeenCalledWith(['explore', 3], { queryParams: component.queryParams });
  });
  it('should call getChannelId method', () => {
    const orgService = TestBed.get(OrgDetailsService);
    spyOn(orgService, 'getOrgDetails').and.callFake(() => observableOf(Response.orgDetailsSuccessData));
    component.getChannelId();
    expect(component.hashTagId).toEqual(Response.orgDetailsSuccessData.hashTagId);
  });
  it('should call getChannelId method and return error', () => {
    const router = TestBed.get(Router);
    const orgService = TestBed.get(OrgDetailsService);
    const response = {};
    spyOn(orgService, 'getOrgDetails').and.callFake(() => observableThrowError(response));
    spyOn(component, 'getChannelId').and.callThrough();
    component.getChannelId();
    expect(router.navigate).toHaveBeenCalledWith(['']);
  });
  it('should call playContent method in Public player service when triggered with appropriate arguments', () => {
    const router = TestBed.get(Router);
    const publicPlayerService = TestBed.get(PublicPlayerService);
    component.queryParams = {};
    const event = {
      data: {
        metaData:
          { mimeType: 'application/vnd.ekstep.content-collection', identifier: '1234' }
      }
    };
    spyOn(publicPlayerService, 'playContent');
    component.playContent(event);
    expect(publicPlayerService.playContent).toHaveBeenCalledWith(event);
  });
  it('should call inview method', () => {
    component.telemetryImpression = {
      context: {
        env: 'public'
      },
      edata: {
        type: '',
        pageid: '',
        uri: '',
        subtype: 'pageexit'
      }
    };
    const event = Response.inviewData;
    component.inview(event);
    expect(component.telemetryImpression.edata.subtype).toEqual('pageexit');
  });
  it('should unsubscribe from all observable subscriptions', () => {
    spyOn(component.unsubscribe$, 'complete');
    component.ngOnDestroy();
    expect(component.unsubscribe$.complete).toHaveBeenCalled();
  });
  it('should call getFilters with data', () => {
    const searchService = TestBed.get(SearchService);
    const filters = Response.filters;
    const requestParams = Response.requestParam2;
    const config = TestBed.get(ConfigService);
    requestParams['params'] = config.appConfig.ExplorePage.contentApiQueryParams;
    component.dataDrivenFilter = {};
    component.hashTagId =   '0123166367624478721';
    spyOn(component, 'populateContentSearch').and.callThrough();
    spyOn(searchService, 'contentSearch').and.callThrough();
    component.getFilters(filters);
    expect(component.facetArray).toEqual([ 'board', 'medium', 'subject', 'gradeLevel' ]);
    expect(component.dataDrivenFilter['board']).toBe('CBSE');
    expect(component.populateContentSearch).toHaveBeenCalled();
    expect(searchService.contentSearch).toHaveBeenCalledWith(requestParams);
  });
  it('should call getFilters with no data', () => {
    const searchService = TestBed.get(SearchService);
    const filters = [];
    const config = TestBed.get(ConfigService);
    const requestParams = Response.requestParam3;
    requestParams['params'] = config.appConfig.ExplorePage.contentApiQueryParams;
    component.dataDrivenFilter = {};
    component.hashTagId =   '0123166367624478721';
    spyOn(component, 'populateContentSearch').and.callThrough();
    spyOn(searchService, 'contentSearch').and.callThrough();
    component.getFilters(filters);
    expect(component.populateContentSearch).toHaveBeenCalled();
    expect(searchService.contentSearch).toHaveBeenCalledWith(requestParams);
  });
});
